@@ -14,7 +14,12 @@ import logging
 import numpy as np
 import pyomo.core as po  # pylint: disable=import-error
 
-from calliope.backend.pyomo.util import loc_tech_is_in, get_param, invalid
+from calliope.backend.pyomo.util import (
+    loc_tech_is_in,
+    get_param,
+    invalid,
+    get_timestep_weight,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -73,7 +78,6 @@ def load_constraints(backend_model):
                 "group_carrier_prod_{}_constraint".format(sense),
                 po.Constraint(
                     getattr(backend_model, "group_names_carrier_prod_{}".format(sense)),
-                    backend_model.carriers,
                     [sense],
                     rule=carrier_prod_constraint_rule,
                 ),
@@ -85,7 +89,6 @@ def load_constraints(backend_model):
                 "group_carrier_con_{}_constraint".format(sense),
                 po.Constraint(
                     getattr(backend_model, "group_names_carrier_con_{}".format(sense)),
-                    backend_model.carriers,
                     [sense],
                     rule=carrier_con_constraint_rule,
                 ),
@@ -97,7 +100,6 @@ def load_constraints(backend_model):
                 "group_demand_share_{}_constraint".format(sense),
                 po.Constraint(
                     getattr(backend_model, "group_names_demand_share_{}".format(sense)),
-                    backend_model.carriers,
                     [sense],
                     rule=demand_share_constraint_rule,
                 ),
@@ -112,7 +114,6 @@ def load_constraints(backend_model):
                         backend_model,
                         "group_names_demand_share_per_timestep_{}".format(sense),
                     ),
-                    backend_model.carriers,
                     backend_model.timesteps,
                     [sense],
                     rule=demand_share_per_timestep_constraint_rule,
@@ -127,7 +128,6 @@ def load_constraints(backend_model):
                     getattr(
                         backend_model, "group_names_carrier_prod_share_{}".format(sense)
                     ),
-                    backend_model.carriers,
                     [sense],
                     rule=carrier_prod_share_constraint_rule,
                 ),
@@ -142,7 +142,6 @@ def load_constraints(backend_model):
                         backend_model,
                         "group_names_carrier_prod_share_per_timestep_{}".format(sense),
                     ),
-                    backend_model.carriers,
                     backend_model.timesteps,
                     [sense],
                     rule=carrier_prod_share_per_timestep_constraint_rule,
@@ -157,7 +156,6 @@ def load_constraints(backend_model):
                     getattr(
                         backend_model, "group_names_net_import_share_{}".format(sense)
                     ),
-                    backend_model.carriers,
                     [sense],
                     rule=net_import_share_constraint_rule,
                 ),
@@ -200,17 +198,19 @@ def load_constraints(backend_model):
             )
 
     if "group_demand_share_per_timestep_decision" in model_data_dict:
-        backend_model.group_demand_share_per_timestep_decision_main_constraint = po.Constraint(
-            backend_model.group_names_demand_share_per_timestep_decision,
-            backend_model.carriers,
-            backend_model.techs,
-            backend_model.timesteps,
-            rule=demand_share_per_timestep_decision_main_constraint_rule,
+        backend_model.group_demand_share_per_timestep_decision_main_constraint = (
+            po.Constraint(
+                backend_model.group_names_demand_share_per_timestep_decision,
+                backend_model.loc_tech_carriers_demand_share_per_timestep_lhs,
+                backend_model.timesteps,
+                rule=demand_share_per_timestep_decision_main_constraint_rule,
+            )
         )
-        backend_model.group_demand_share_per_timestep_decision_sum_constraint = po.Constraint(
-            backend_model.group_names_demand_share_per_timestep_decision,
-            backend_model.carriers,
-            rule=demand_share_per_timestep_decision_sum_constraint_rule,
+        backend_model.group_demand_share_per_timestep_decision_sum_constraint = (
+            po.Constraint(
+                backend_model.group_names_demand_share_per_timestep_decision,
+                rule=demand_share_per_timestep_decision_sum_constraint_rule,
+            )
         )
 
 
@@ -225,29 +225,23 @@ def equalizer(lhs, rhs, sign):
         raise ValueError("Invalid sign: {}".format(sign))
 
 
-def get_demand_share_lhs_and_rhs_loc_tech_carriers(backend_model, group_name, carrier):
-    # Returns
-    # -------
-    # (lhs_loc_tech_carriers, rhs_loc_tech_carriers):
-    # lhs are the supply technologies, rhs are the demand technologies
-    lhs_loc_techs = getattr(
-        backend_model, "group_constraint_loc_techs_{}".format(group_name)
+def get_group_lhs_loc_tech_carriers(backend_model, group_name):
+    return getattr(
+        backend_model, "group_constraint_loc_tech_carriers_{}".format(group_name)
     )
-    lhs_locs = set(loc_tech.split("::")[0] for loc_tech in lhs_loc_techs)
-    lhs_loc_tech_carriers = [
-        i
-        for i in backend_model.loc_tech_carriers_prod
-        if i.rsplit("::", 1)[0] in lhs_loc_techs and i.split("::")[-1] == carrier
-    ]
-    rhs_loc_tech_carriers = [
-        i
-        for i in backend_model.loc_tech_carriers_demand
-        if i.split("::")[0] in lhs_locs and i.split("::")[-1] == carrier
-    ]
-    return (lhs_loc_tech_carriers, rhs_loc_tech_carriers)
 
 
-def demand_share_constraint_rule(backend_model, group_name, carrier, what):
+def get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name):
+    lhs_loc_tech_carriers = getattr(
+        backend_model, "group_constraint_loc_tech_carriers_{}_lhs".format(group_name)
+    )
+    rhs_loc_tech_carriers = getattr(
+        backend_model, "group_constraint_loc_tech_carriers_{}_rhs".format(group_name)
+    )
+    return lhs_loc_tech_carriers, rhs_loc_tech_carriers
+
+
+def demand_share_constraint_rule(backend_model, group_name, what):
     """
     Enforces shares of demand of a carrier to be met by the given groups
     of technologies at the given locations, on average over the entire
@@ -262,9 +256,7 @@ def demand_share_constraint_rule(backend_model, group_name, carrier, what):
             carrier_{con}(loc::tech::carrier, timestep)
 
     """
-    share = get_param(
-        backend_model, "group_demand_share_{}".format(what), (carrier, group_name)
-    )
+    share = get_param(backend_model, "group_demand_share_{}".format(what), (group_name))
 
     if invalid(share):
         return return_noconstraint("demand_share", group_name)
@@ -272,9 +264,7 @@ def demand_share_constraint_rule(backend_model, group_name, carrier, what):
         (
             lhs_loc_tech_carriers,
             rhs_loc_tech_carriers,
-        ) = get_demand_share_lhs_and_rhs_loc_tech_carriers(
-            backend_model, group_name, carrier
-        )
+        ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
 
         lhs = sum(
             backend_model.carrier_prod[loc_tech_carrier, timestep]
@@ -296,7 +286,7 @@ def demand_share_constraint_rule(backend_model, group_name, carrier, what):
 
 
 def demand_share_per_timestep_constraint_rule(
-    backend_model, group_name, carrier, timestep, what
+    backend_model, group_name, timestep, what
 ):
     """
     Enforces shares of demand of a carrier to be met by the given groups
@@ -315,7 +305,7 @@ def demand_share_per_timestep_constraint_rule(
     share = get_param(
         backend_model,
         "group_demand_share_per_timestep_{}".format(what),
-        (carrier, group_name),
+        (group_name),
     )
 
     if invalid(share):
@@ -324,9 +314,7 @@ def demand_share_per_timestep_constraint_rule(
         (
             lhs_loc_tech_carriers,
             rhs_loc_tech_carriers,
-        ) = get_demand_share_lhs_and_rhs_loc_tech_carriers(
-            backend_model, group_name, carrier
-        )
+        ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
 
         lhs = sum(
             backend_model.carrier_prod[loc_tech_carrier, timestep]
@@ -346,7 +334,7 @@ def demand_share_per_timestep_constraint_rule(
 
 
 def demand_share_per_timestep_decision_main_constraint_rule(
-    backend_model, group_name, carrier, tech, timestep
+    backend_model, group_name, loc_tech_carrier, timestep
 ):
     """
     Allows the model to decide on how a fraction demand for a carrier is met
@@ -373,87 +361,55 @@ def demand_share_per_timestep_decision_main_constraint_rule(
             \\forall tech \\in techs
 
     """
-    share_of_carrier_demand = get_param(
-        backend_model, "group_demand_share_per_timestep_decision", (carrier, group_name)
+    (
+        _,
+        rhs_loc_tech_carriers,
+    ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
+    lhs = backend_model.carrier_prod[loc_tech_carrier, timestep]
+    rhs = (
+        -1
+        * sum(
+            backend_model.required_resource[
+                rhs_loc_tech_carrier.rsplit("::", 1)[0], timestep
+            ]
+            for rhs_loc_tech_carrier in rhs_loc_tech_carriers
+        )
+        * backend_model.demand_share_per_timestep_decision[loc_tech_carrier]
     )
 
-    if invalid(share_of_carrier_demand):
-        return return_noconstraint(
-            "demand_share_per_timestep_decision_main", group_name
-        )
-    else:
-        # lhs are the supply technologies, rhs are the demand technologies
-        (
-            lhs_loc_tech_carriers,
-            rhs_loc_tech_carriers,
-        ) = get_demand_share_lhs_and_rhs_loc_tech_carriers(
-            backend_model, group_name, carrier
-        )
-        # Filter the supply loc_tech_carriers by the current tech
-        lhs_loc_tech_carriers = [
-            i for i in lhs_loc_tech_carriers if "::{}::".format(tech) in i
-        ]
-
-        # Only techs that are in the given group are considered
-        if len(lhs_loc_tech_carriers) == 0:
-            return return_noconstraint(
-                "demand_share_per_timestep_decision_main", group_name
-            )
-
-        lhs = sum(
-            backend_model.carrier_prod[loc_tech_carrier, timestep]
-            for loc_tech_carrier in lhs_loc_tech_carriers
-        )
-
-        rhs = (
-            -1
-            * sum(
-                backend_model.required_resource[
-                    rhs_loc_tech_carrier.rsplit("::", 1)[0], timestep
-                ]
-                for rhs_loc_tech_carrier in rhs_loc_tech_carriers
-            )
-            * sum(
-                backend_model.demand_share_per_timestep_decision[lhs_loc_tech_carrier]
-                for lhs_loc_tech_carrier in lhs_loc_tech_carriers
-            )
-        )
-
-        return equalizer(lhs, rhs, "equals")
+    return equalizer(lhs, rhs, "min")
 
 
-def demand_share_per_timestep_decision_sum_constraint_rule(
-    backend_model, group_name, carrier
-):
+def demand_share_per_timestep_decision_sum_constraint_rule(backend_model, group_name):
     """
-    Allows the model to decide on how a fraction of demand for a carrier is met
-    by the given groups, which will all have the same share in each timestep.
-    The share is relative to the actual demand from ``demand`` technologies only.
+       Allows the model to decide on how a fraction of demand for a carrier is met
+       by the given groups, which will all have the same share in each timestep.
+       The share is relative to the actual demand from ``demand`` technologies only.
 
-    The sum constraint ensures that all decision shares add up to the share of
-    carrier demand specified in the constraint.
+       The sum constraint ensures that all decision shares add up to the share of
+       carrier demand specified in the constraint.
 
-    This constraint is only applied if the share of carrier demand has been
-    set to a not-None value.
+       This constraint is only applied if the share of carrier demand has been
+       set to a not-None value.
 
- .. container:: scrolling-wrapper
+    .. container:: scrolling-wrapper
 
-        .. math::
-            share = \\sum_{loc::tech::carrier \\in given\\_group}
-            demand\\_share\\_per\\_timestep\\_decision(loc::tech::carrier)
+           .. math::
+               share = \\sum_{loc::tech::carrier \\in given\\_group}
+               demand\\_share\\_per\\_timestep\\_decision(loc::tech::carrier)
 
 
     """
     share_of_carrier_demand = get_param(
-        backend_model, "group_demand_share_per_timestep_decision", (carrier, group_name)
+        backend_model, "group_demand_share_per_timestep_decision", (group_name)
     )
 
     # If inf was given that means that we don't limit the total share
-    if invalid(share_of_carrier_demand) or np.isinf(share_of_carrier_demand):
+    if invalid(share_of_carrier_demand) or np.isinf(po.value(share_of_carrier_demand)):
         return return_noconstraint("demand_share_per_timestep_decision_sum", group_name)
     else:
-        lhs_loc_tech_carriers, _ = get_demand_share_lhs_and_rhs_loc_tech_carriers(
-            backend_model, group_name, carrier
+        lhs_loc_tech_carriers, _ = get_group_lhs_and_rhs_loc_tech_carriers(
+            backend_model, group_name
         )
 
         return share_of_carrier_demand == sum(
@@ -462,24 +418,7 @@ def demand_share_per_timestep_decision_sum_constraint_rule(
         )
 
 
-def get_carrier_lhs_loc_techs(backend_model, group_name):
-    lhs_loc_techs = getattr(
-        backend_model, "group_constraint_loc_techs_{}".format(group_name)
-    )
-    return lhs_loc_techs
-
-
-def get_carrier_prod_share_rhs_loc_techs(backend_model, lhs_loc_techs):
-    lhs_locs = [loc_tech.split("::")[0] for loc_tech in lhs_loc_techs]
-    rhs_loc_techs = [
-        i
-        for i in backend_model.loc_techs_supply_conversion_all
-        if i.split("::")[0] in lhs_locs
-    ]
-    return rhs_loc_techs
-
-
-def carrier_prod_share_constraint_rule(backend_model, constraint_group, carrier, what):
+def carrier_prod_share_constraint_rule(backend_model, group_name, what):
     """
     Enforces shares of carrier_prod for groups of technologies and locations,
     on average over the entire model period. The share is relative to
@@ -495,27 +434,25 @@ def carrier_prod_share_constraint_rule(backend_model, constraint_group, carrier,
 
     """
     share = get_param(
-        backend_model,
-        "group_carrier_prod_share_{}".format(what),
-        (carrier, constraint_group),
+        backend_model, "group_carrier_prod_share_{}".format(what), (group_name)
     )
 
     if invalid(share):
-        return return_noconstraint("carrier_prod_share", constraint_group)
+        return return_noconstraint("carrier_prod_share", group_name)
     else:
-        lhs_loc_techs = get_carrier_lhs_loc_techs(backend_model, constraint_group)
-        rhs_loc_techs = get_carrier_prod_share_rhs_loc_techs(
-            backend_model, lhs_loc_techs
-        )
+        (
+            lhs_loc_tech_carriers,
+            rhs_loc_tech_carriers,
+        ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
 
         lhs = sum(
-            backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-            for loc_tech in lhs_loc_techs
+            backend_model.carrier_prod[loc_tech_carrier, timestep]
+            for loc_tech_carrier in lhs_loc_tech_carriers
             for timestep in backend_model.timesteps
         )
         rhs = share * sum(
-            backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-            for loc_tech in rhs_loc_techs
+            backend_model.carrier_prod[loc_tech_carrier, timestep]
+            for loc_tech_carrier in rhs_loc_tech_carriers
             for timestep in backend_model.timesteps
         )
 
@@ -523,7 +460,7 @@ def carrier_prod_share_constraint_rule(backend_model, constraint_group, carrier,
 
 
 def carrier_prod_share_per_timestep_constraint_rule(
-    backend_model, constraint_group, carrier, timestep, what
+    backend_model, group_name, timestep, what
 ):
     """
     Enforces shares of carrier_prod for groups of technologies and locations,
@@ -542,30 +479,30 @@ def carrier_prod_share_per_timestep_constraint_rule(
     share = get_param(
         backend_model,
         "group_carrier_prod_share_per_timestep_{}".format(what),
-        (carrier, constraint_group),
+        (group_name),
     )
 
     if invalid(share):
-        return return_noconstraint("carrier_prod_share_per_timestep", constraint_group)
+        return return_noconstraint("carrier_prod_share_per_timestep", group_name)
     else:
-        lhs_loc_techs = get_carrier_lhs_loc_techs(backend_model, constraint_group)
-        rhs_loc_techs = get_carrier_prod_share_rhs_loc_techs(
-            backend_model, lhs_loc_techs
-        )
+        (
+            lhs_loc_tech_carriers,
+            rhs_loc_tech_carriers,
+        ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
 
         lhs = sum(
-            backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-            for loc_tech in lhs_loc_techs
+            backend_model.carrier_prod[loc_tech_carrier, timestep]
+            for loc_tech_carrier in lhs_loc_tech_carriers
         )
         rhs = share * sum(
-            backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-            for loc_tech in rhs_loc_techs
+            backend_model.carrier_prod[loc_tech_carrier, timestep]
+            for loc_tech_carrier in rhs_loc_tech_carriers
         )
 
         return equalizer(lhs, rhs, what)
 
 
-def net_import_share_constraint_rule(backend_model, constraint_group, carrier, what):
+def net_import_share_constraint_rule(backend_model, group_name, what):
     """
     Enforces demand shares of net imports from transmission technologies for groups of locations,
     on average over the entire model period. Transmission within the group are ignored. The share
@@ -581,45 +518,33 @@ def net_import_share_constraint_rule(backend_model, constraint_group, carrier, w
             carrier_{con}(loc::tech::carrier, timestep)
 
     """
-    share = get_param(
-        backend_model,
-        "group_net_import_share_{}".format(what),
-        (carrier, constraint_group),
-    )
+    share = get_param(backend_model, f"group_net_import_share_{what}", (group_name))
 
     if invalid(share):
-        return return_noconstraint("net_import_share", constraint_group)
+        return return_noconstraint("net_import_share", group_name)
     else:
-        trans_loc_tech = getattr(
-            backend_model, "group_constraint_loc_techs_{}".format(constraint_group)
-        )
-        locs = set(loc_tech.split("::")[0] for loc_tech in trans_loc_tech)
-        trans_loc_tech = filter(
-            lambda loc_tec: loc_tec.split(":")[-1] not in locs, trans_loc_tech
-        )
-        demand_loc_tech = [
-            i
-            for i in backend_model.loc_tech_carriers_demand
-            if i.split("::")[0] in locs
-        ]
+        (
+            lhs_loc_tech_carriers,
+            rhs_loc_tech_carriers,
+        ) = get_group_lhs_and_rhs_loc_tech_carriers(backend_model, group_name)
 
         lhs = sum(
             (
-                backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-                + backend_model.carrier_con[loc_tech + "::" + carrier, timestep]
+                backend_model.carrier_prod[loc_tech_carrier, timestep]
+                + backend_model.carrier_con[loc_tech_carrier, timestep]
             )
-            for loc_tech in trans_loc_tech
+            for loc_tech_carrier in lhs_loc_tech_carriers
             for timestep in backend_model.timesteps
         )
         rhs = -share * sum(
-            backend_model.carrier_con[loc_tech, timestep]
-            for loc_tech in demand_loc_tech
+            backend_model.carrier_con[loc_tech_carrier, timestep]
+            for loc_tech_carrier in rhs_loc_tech_carriers
             for timestep in backend_model.timesteps
         )
         return equalizer(lhs, rhs, what)
 
 
-def carrier_prod_constraint_rule(backend_model, constraint_group, carrier, what):
+def carrier_prod_constraint_rule(backend_model, group_name, what):
     """
     Enforces carrier_prod for groups of technologies and locations,
     as a sum over the entire model period.
@@ -631,25 +556,24 @@ def carrier_prod_constraint_rule(backend_model, constraint_group, carrier, what)
             \\sum_{loc::tech::carrier \\in given\\_group, timestep \\in timesteps} carrier_{prod}(loc::tech::carrier, timestep) \\leq carrier_prod_max
 
     """
-    limit = get_param(
-        backend_model, "group_carrier_prod_{}".format(what), (carrier, constraint_group)
-    )
+    limit = get_param(backend_model, f"group_carrier_prod_{what}", (group_name))
 
     if invalid(limit):
-        return return_noconstraint("carrier_prod", constraint_group)
+        return return_noconstraint("carrier_prod", group_name)
     else:
-        lhs_loc_techs = get_carrier_lhs_loc_techs(backend_model, constraint_group)
+        lhs_loc_tech_carriers = get_group_lhs_loc_tech_carriers(
+            backend_model, group_name
+        )
 
         lhs = sum(
-            backend_model.carrier_prod[loc_tech + "::" + carrier, timestep]
-            for loc_tech in lhs_loc_techs
+            backend_model.carrier_prod[loc_tech_carrier, timestep]
+            for loc_tech_carrier in lhs_loc_tech_carriers
             for timestep in backend_model.timesteps
-            if loc_tech + "::" + carrier in backend_model.loc_tech_carriers_prod
         )
         return equalizer(lhs, limit, what)
 
 
-def carrier_con_constraint_rule(backend_model, constraint_group, carrier, what):
+def carrier_con_constraint_rule(backend_model, constraint_group, what):
     """
     Enforces carrier_con for groups of technologies and locations,
     as a sum over the entire model period. limits are always negative, so min/max
@@ -663,19 +587,20 @@ def carrier_con_constraint_rule(backend_model, constraint_group, carrier, what):
 
     """
     limit = get_param(
-        backend_model, "group_carrier_con_{}".format(what), (carrier, constraint_group)
+        backend_model, "group_carrier_con_{}".format(what), (constraint_group)
     )
 
     if invalid(limit):
         return return_noconstraint("carrier_con", constraint_group)
     else:
-        lhs_loc_techs = get_carrier_lhs_loc_techs(backend_model, constraint_group)
+        lhs_loc_tech_carriers = get_group_lhs_loc_tech_carriers(
+            backend_model, constraint_group
+        )
 
         lhs = sum(
-            backend_model.carrier_con[loc_tech + "::" + carrier, timestep]
-            for loc_tech in lhs_loc_techs
+            backend_model.carrier_con[loc_tech_carrier, timestep]
+            for loc_tech_carrier in lhs_loc_tech_carriers
             for timestep in backend_model.timesteps
-            if loc_tech + "::" + carrier in backend_model.loc_tech_carriers_con
         )
 
         return equalizer(limit, lhs, what)
@@ -684,7 +609,8 @@ def carrier_con_constraint_rule(backend_model, constraint_group, carrier, what):
 def energy_cap_share_constraint_rule(backend_model, constraint_group, what):
     """
     Enforces shares of energy_cap for groups of technologies and locations. The
-    share is relative to ``supply`` and ``supply_plus`` technologies only.
+    share is relative to ``supply``, ``supply_plus``, ``conversion``, and ``conversion_plus``
+    technologies only.
 
     .. container:: scrolling-wrapper
 
@@ -758,6 +684,37 @@ def energy_cap_constraint_rule(backend_model, constraint_group, what):
         return equalizer(sum(lhs), rhs, what)
 
 
+def storage_cap_constraint_rule(backend_model, constraint_group, what):
+    """
+    Enforce upper and lower bounds for storage_cap of storage_cap
+    for groups of technologies and locations.
+
+    .. container:: scrolling-wrapper
+
+        .. math::
+
+            \\sum_{loc::tech \\in given\\_group} storage_{cap}(loc::tech) \\leq storage\\_cap\\_max\\\\
+
+            \\sum_{loc::tech \\in given\\_group} storage_{cap}(loc::tech) \\geq storage\\_cap\\_min
+
+    """
+    threshold = get_param(
+        backend_model, "group_storage_cap_{}".format(what), (constraint_group)
+    )
+
+    if invalid(threshold):
+        return return_noconstraint("storage_cap", constraint_group)
+    else:
+        lhs_loc_techs = getattr(
+            backend_model, "group_constraint_loc_techs_{}".format(constraint_group)
+        )
+
+        lhs = sum(backend_model.storage_cap[loc_tech] for loc_tech in lhs_loc_techs)
+        rhs = threshold
+
+        return equalizer(lhs, rhs, what)
+
+
 def cost_cap_constraint_rule(backend_model, group_name, cost, what):
     """
     Limit cost for a specific cost class to a certain value,
@@ -780,7 +737,6 @@ def cost_cap_constraint_rule(backend_model, group_name, cost, what):
     cost_cap = get_param(
         backend_model, "group_cost_{}".format(what), (cost, group_name)
     )
-
     if invalid(cost_cap):
         return return_noconstraint("cost_cap", group_name)
     else:
