@@ -6,17 +6,17 @@ Licensed under the Apache 2.0 License (see LICENSE file).
 import logging
 
 import numpy as np
+import pandas as pd
 import xarray as xr
-
-from calliope.core.util.logging import log_time
 from calliope import exceptions
 from calliope.backend import checks
-from calliope.backend.pyomo import model as run_pyomo
 from calliope.backend.pyomo import interface as pyomo_interface
-
-from calliope.core.util.observed_dict import UpdateObserverDict
+from calliope.backend.pyomo import model as run_pyomo
+from calliope.core import io
 from calliope.core.attrdict import AttrDict
 from calliope.core.util.dataset import split_loc_techs
+from calliope.core.util.logging import log_time
+from calliope.core.util.observed_dict import UpdateObserverDict
 
 logger = logging.getLogger(__name__)
 
@@ -43,7 +43,7 @@ def run(model_data, timings, build_only=False):
     run_config = AttrDict.from_yaml_string(model_data.attrs["run_config"])
 
     if run_config["mode"] == "plan":
-        results, backend = run_plan(
+        results, backend, opt = run_plan(
             model_data,
             timings,
             backend=BACKEND[run_config.backend],
@@ -51,7 +51,7 @@ def run(model_data, timings, build_only=False):
         )
 
     elif run_config["mode"] == "operate":
-        results, backend = run_operate(
+        results, backend, opt = run_operate(
             model_data,
             timings,
             backend=BACKEND[run_config.backend],
@@ -59,7 +59,7 @@ def run(model_data, timings, build_only=False):
         )
 
     elif run_config["mode"] == "spores":
-        results, backend = run_spores(
+        results, backend, opt = run_spores(
             model_data,
             timings,
             interface=INTERFACE[run_config.backend],
@@ -67,16 +67,25 @@ def run(model_data, timings, build_only=False):
             build_only=build_only,
         )
 
-    return results, backend, INTERFACE[run_config.backend].BackendInterfaceMethods
+    return results, backend, opt, INTERFACE[run_config.backend].BackendInterfaceMethods
 
 
-def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
+def run_plan(
+    model_data,
+    timings,
+    backend,
+    build_only,
+    backend_rerun=False,
+    allow_warmstart=False,
+    persistent=True,
+    opt=None,
+):
 
     log_time(logger, timings, "run_start", comment="Backend: starting model run")
 
+    warmstart = False
     if not backend_rerun:
         backend_model = backend.generate_model(model_data)
-
         log_time(
             logger,
             timings,
@@ -87,6 +96,8 @@ def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
 
     else:
         backend_model = backend_rerun
+        if allow_warmstart:
+            warmstart = True
 
     run_config = backend_model.__calliope_run_config
     solver = run_config["solver"]
@@ -98,6 +109,12 @@ def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
         results = xr.Dataset()
 
     else:
+        if "persistent" in solver and persistent is False:
+            exceptions.warn(
+                f"The chosen solver, `{solver}` will not be used in this run. "
+                f"`{solver.replace('_persistent', '')}` will be used instead."
+            )
+            solver = solver.replace("_persistent", "")
         log_time(
             logger,
             timings,
@@ -105,12 +122,14 @@ def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
             comment="Backend: sending model to solver",
         )
 
-        results = backend.solve_model(
+        backend_results, opt = backend.solve_model(
             backend_model,
             solver=solver,
             solver_io=solver_io,
             solver_options=solver_options,
             save_logs=save_logs,
+            warmstart=warmstart,
+            opt=opt,
         )
 
         log_time(
@@ -121,17 +140,21 @@ def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
             comment="Backend: solver finished running",
         )
 
-        termination = backend.load_results(backend_model, results)
+        termination = backend.load_results(backend_model, backend_results, opt)
 
         log_time(
             logger, timings, "run_results_loaded", comment="Backend: loaded results"
         )
 
-        results = backend.get_result_array(backend_model, model_data)
-        results.attrs["termination_condition"] = termination
-
-        if results.attrs["termination_condition"] in ["optimal", "feasible"]:
-            results.attrs["objective_function_value"] = backend_model.obj()
+        if termination in ["optimal", "feasible"]:
+            results = backend.get_result_array(backend_model, model_data)
+            results.attrs["termination_condition"] = termination
+            if "persistent" in opt.name and persistent is True:
+                results.attrs["objective_function_value"] = opt.get_model_attr("ObjVal")
+            else:
+                results.attrs["objective_function_value"] = backend_model.obj()
+        else:
+            results = xr.Dataset(attrs={"termination_condition": termination})
 
         log_time(
             logger,
@@ -141,10 +164,12 @@ def run_plan(model_data, timings, backend, build_only, backend_rerun=False):
             comment="Backend: generated solution array",
         )
 
-    return results, backend_model
+    return results, backend_model, opt
 
 
-def run_spores(model_data, timings, interface, backend, build_only):
+def run_spores(
+    model_data, timings, interface, backend, build_only, backend_rerun=False, opt=None
+):
     """
     For use when mode is 'spores', to allow the model to be built, edited, and
     iteratively run within Pyomo, modifying, at each iteration, the score of
@@ -159,48 +184,21 @@ def run_spores(model_data, timings, interface, backend, build_only):
         comment="Backend: starting model run in SPORES mode",
     )
 
-    run_config = UpdateObserverDict(
-        initial_yaml_string=model_data.attrs["run_config"],
-        name="run_config",
-        observer=model_data,
-    )
-
-    backend_model = backend.generate_model(model_data)
-
-    log_time(
-        logger,
-        timings,
-        "run_backend_model_generated",
-        time_since_run_start=True,
-        comment="Backend: model generated",
-    )
-
-    n_spores = run_config["spores_options"]["spores_number"]
-    slack = run_config["spores_options"]["slack"]
-    spores_score = run_config["spores_options"]["score_cost_class"]
-    slack_group = run_config["spores_options"]["slack_cost_group"]
-
-    # Define default scoring function, based on integer scoring method
-    # TODO: make the function to run optional
-    def _cap_loc_score_default(results, subset=None):
-        if subset is None:
-            subset = {}
-        cap_loc_score = split_loc_techs(results["energy_cap"]).loc[subset]
-        cap_loc_score = cap_loc_score.where(cap_loc_score > 1e-3, other=0)
-        cap_loc_score = cap_loc_score.where(cap_loc_score == 0, other=100)
-
-        return cap_loc_score.to_pandas()
+    def _cap_loc_score_default(results):
+        # Define default scoring function, based on integer scoring method
+        # TODO: make it possible to point to a custom function instead of this one
+        cap_loc_score = xr.where(results.energy_cap > 1e-3, 100, 0).loc[
+            {"loc_techs": results.loc_techs_investment_cost}
+        ]
+        return cap_loc_score.to_series().rename_axis(index="loc_techs_investment_cost")
 
     # Define function to update "spores_score" after each iteration of the method
     def _update_spores_score(backend_model, cap_loc_score):
-        loc_tech_score_dict = {
-            (spores_score, "{}::{}".format(i, j)): k
-            for (i, j), k in cap_loc_score.stack().items()
-            if "{}::{}".format(i, j) in model_data.loc_techs_investment_cost
-        }
+        print("Updating loc::tech spores scores")
+        loc_tech_score_dict = cap_loc_score.to_dict()
 
         interface.update_pyomo_param(
-            backend_model, "cost_energy_cap", loc_tech_score_dict
+            backend_model, opt, "cost_energy_cap", loc_tech_score_dict
         )
 
     def _warn_on_infeasibility():
@@ -209,64 +207,199 @@ def run_spores(model_data, timings, interface, backend, build_only):
             "No more SPORES will be generated."
         )
 
-    # Run once for the 'cost-optimal' solution
-    results, backend_model = run_plan(model_data, timings, backend, build_only)
-    if build_only:
-        return results, backend_model  # We have what we need, so break out of the loop
+    def _get_updated_spores_inputs(backend_model):
+        inputs = interface.access_pyomo_model_inputs(backend_model)
+        inputs_to_keep = ["cost_energy_cap", "group_cost_max"]
+        return inputs[inputs_to_keep]
 
-    if results.attrs["termination_condition"] in ["optimal", "feasible"]:
-        results.attrs["objective_function_value"] = backend_model.obj()
-        # Storing results and scores in the specific dictionaries
-        spores_list = [results]
-        cum_scores = _cap_loc_score_default(results)
-        # Set group constraint "cost_max" equal to slacked cost
+    def _combine_spores_results_and_inputs(
+        backend_model, results, spore_num, model_data=None
+    ):
+        inputs_to_keep = _get_updated_spores_inputs(backend_model)
+        for var_data in results.data_vars.values():
+            if "is_result" not in var_data.attrs.keys():
+                var_data.attrs["is_result"] = 1
+        if model_data is not None:
+            datasets = [inputs_to_keep, model_data, results]
+        else:
+            datasets = [inputs_to_keep, results]
+        new_ds = xr.combine_by_coords(
+            datasets, compat="override", combine_attrs="no_conflicts"
+        )
+        return new_ds.assign_coords(spores=("spores", [spore_num]))
+
+    def _add_results_to_list(backend_model, spores_results, results, spore_num):
+        results_to_add = _combine_spores_results_and_inputs(
+            backend_model, results, spore_num
+        )
+        spores_results[spore_num] = results_to_add
+
+    def _save_spore(backend_model, results, spore_num, model_data=None):
+        _path = spores_config["save_per_spore_path"].format(spore_num)
+        new_ds = _combine_spores_results_and_inputs(
+            backend_model, results, spore_num, model_data
+        )
+        print(f"Saving SPORE {spore_num} to {_path}")
+        io.save_netcdf(new_ds, _path)
+
+    def _update_slack_cost_constraint(backend_model):
         slack_costs = model_data.group_cost_max.loc[
-            {"group_names_cost_max": slack_group}
+            {"group_names_cost_max": spores_config["slack_cost_group"]}
         ].dropna("costs")
         interface.update_pyomo_param(
             backend_model,
+            opt,
             "group_cost_max",
             {
-                (_cost_class, slack_group): results.cost.loc[{"costs": _cost_class}]
+                (_cost_class, spores_config["slack_cost_group"]): results.cost.loc[
+                    {"costs": _cost_class}
+                ]
                 .sum()
                 .item()
-                * (1 + slack)
+                * (1 + spores_config["slack"])
                 for _cost_class in slack_costs.costs.values
             },
         )
-        # Modify objective function weights: spores_score -> 1, all others -> 0
+
+    def _update_to_spores_objective(backend_model):
         interface.update_pyomo_param(
             backend_model,
+            opt,
             "objective_cost_class",
-            {spores_score: 1, **{i: 0 for i in slack_costs.costs.values}},
+            spores_config["objective_cost_class"],
         )
-        # Update "spores_score" based on previous iteration
-        _update_spores_score(backend_model, cum_scores)
-    else:
-        _warn_on_infeasibility()
-        return results, backend_model
 
-    log_time(
-        logger,
-        timings,
-        "run_solution_returned",
-        time_since_run_start=True,
-        comment="Backend: generated solution array for the cost-optimal case",
+    def _initialise_backend_model():
+        if backend_rerun:
+            kwargs = {"backend_rerun": backend_rerun, "opt": opt}
+        else:
+            kwargs = {}
+        if spores_config["skip_cost_op"]:
+            print(
+                "Skipping cost optimal run and using model_data to initialise SPORES directly"
+            )
+            return run_plan(model_data, timings, backend, build_only=True, **kwargs)
+        else:
+            # Run once for the 'cost-optimal' solution
+            return run_plan(
+                model_data, timings, backend, build_only, persistent=False, **kwargs
+            )
+
+    def _initialise_spores_number():
+        if "spores" in model_data.coords and spores_config["skip_cost_op"]:
+            return model_data.spores.max().item()
+        else:
+            return 0
+
+    def _error_on_malformed_input():
+        if backend_rerun:
+            try:
+                backend_rerun.obj()
+            except ValueError:  # model has not yet been run
+                pass
+            else:
+                raise exceptions.ModelError(
+                    "Cannot run SPORES if the backend model already has a solution. "
+                    "Consider using the `build_only` optional `run()` argument to avoid this."
+                )
+        if "spores" in model_data.filter_by_attrs(is_result=0).squeeze().dims:
+            raise exceptions.ModelError(
+                "Cannot run SPORES with a SPORES dimension in any input (e.g. `cost_energy_cap`)."
+            )
+
+    _error_on_malformed_input()
+
+    if backend_rerun:
+        model_data = _combine_spores_results_and_inputs(
+            backend_rerun, xr.Dataset(), 0, model_data=model_data
+        )
+    run_config = UpdateObserverDict(
+        initial_yaml_string=model_data.attrs["run_config"],
+        name="run_config",
+        observer=model_data,
     )
 
+    spores_config = run_config["spores_options"]
+    if "spores" in model_data.dims and model_data.spores.size == 1:
+        model_data = model_data.squeeze("spores")
+
+    init_spores_scores = (
+        model_data.cost_energy_cap.loc[{"costs": [spores_config["score_cost_class"]]}]
+        .to_series()
+        .dropna()
+    )
+    spores_results = {}
+
+    results, backend_model, opt = _initialise_backend_model()
+    if build_only:
+        return results, backend_model, opt
+
+    init_spore = _initialise_spores_number()
+
+    if spores_config["skip_cost_op"]:
+        cumulative_spores_scores = init_spores_scores.copy()
+        _update_to_spores_objective(backend_model)
+    elif results.attrs["termination_condition"] in ["optimal", "feasible"]:
+        results.attrs["objective_function_value"] = backend_model.obj()
+        if spores_config["save_per_spore"] is True:
+            _save_spore(backend_model, results, init_spore, model_data=model_data)
+        # Storing results and scores in the specific dictionaries
+        _add_results_to_list(backend_model, spores_results, results, 0)
+        cumulative_spores_scores = init_spores_scores + _cap_loc_score_default(results)
+        # Set group constraint "cost_max" equal to slacked cost
+        _update_slack_cost_constraint(backend_model)
+        _update_to_spores_objective(backend_model)
+        # Update "spores_score" based on previous iteration
+        _update_spores_score(backend_model, cumulative_spores_scores)
+
+        log_time(
+            logger,
+            timings,
+            "run_solution_returned",
+            time_since_run_start=True,
+            comment="Backend: generated solution array for the cost-optimal case",
+        )
+    else:
+        _warn_on_infeasibility()
+        return results, backend_model, opt
+
     # Iterate over the number of SPORES requested by the user
-    for _spore in range(0, n_spores):
-        results, backend_model = run_plan(
-            model_data, timings, backend, build_only, backend_rerun=backend_model
+    for _spore in range(init_spore + 1, spores_config["spores_number"] + 1):
+        print(f"Running SPORES {_spore}")
+        if opt is not None and "persistent" in opt.type:
+            opt = interface.regenerate_persistent_pyomo_solver(
+                backend_model,
+                opt,
+                obj=True,
+                constraints={
+                    "cost_investment_constraint": cumulative_spores_scores.index,
+                    "cost_constraint": cumulative_spores_scores.index,
+                },
+            )
+        else:
+            opt = None
+        results, backend_model, opt = run_plan(
+            model_data,
+            timings,
+            backend,
+            build_only=False,
+            backend_rerun=backend_model,
+            allow_warmstart=False,
+            persistent=True,
+            opt=opt,
         )
 
         if results.attrs["termination_condition"] in ["optimal", "feasible"]:
             results.attrs["objective_function_value"] = backend_model.obj()
+            if spores_config["save_per_spore"] is True:
+                _save_spore(backend_model, results, _spore)
             # Storing results and scores in the specific dictionaries
-            spores_list.append(results)
-            cum_scores += _cap_loc_score_default(results)
+            _add_results_to_list(backend_model, spores_results, results, _spore)
+            print(f"Updating capacity scores from {cumulative_spores_scores.sum()}...")
+            cumulative_spores_scores += _cap_loc_score_default(results)
+            print(f"... to {cumulative_spores_scores.sum()}")
             # Update "spores_score" based on previous iteration
-            _update_spores_score(backend_model, cum_scores)
+            _update_spores_score(backend_model, cumulative_spores_scores)
         else:
             _warn_on_infeasibility()
             break
@@ -275,14 +408,14 @@ def run_spores(model_data, timings, interface, backend, build_only):
             timings,
             "run_solution_returned",
             time_since_run_start=True,
-            comment="Backend: generated solution array for the cost-optimal case",
+            comment=f"Backend: generated solution array for the SPORE {_spore}",
         )
-        # TODO: make this function work with the spores dimension,
-        # so that postprocessing can take place in core/model.py, as with run_plan and run_operate
 
-    results = xr.concat(spores_list, dim="spores")
+    results = xr.concat(
+        spores_results.values(), dim=pd.Index(spores_results.keys(), name="spores")
+    )
 
-    return results, backend_model
+    return results, backend_model, opt
 
 
 def run_operate(model_data, timings, backend, build_only):
@@ -376,6 +509,9 @@ def run_operate(model_data, timings, backend, build_only):
     horizon_ends = timestep_cumsum[
         timestep_cumsum.isin(window_ends.values + window_to_horizon)
     ]
+    # solves bug where horizon_ends is empty if window < # of timesteps < horizon
+    if len(horizon_ends) == 0:
+        horizon_ends = timestep_cumsum[timestep_cumsum == window_ends.values[-1]]
 
     if not any(window_starts):
         raise exceptions.ModelError(
@@ -422,8 +558,8 @@ def run_operate(model_data, timings, backend, build_only):
             backend_model = backend.generate_model(window_model_data)
 
         # Build the full model in the last instance(s),
-        # where number of timesteps is less than the horizon length
-        elif i > len(horizon_ends) - 1:
+        # where the number of timesteps may be less than the horizon length
+        elif i > len(horizon_ends) - 1 or i == iterations[-1]:
             warmstart = False
             end_timestep = window_ends.index[i]
             timesteps = slice(start_timestep, end_timestep)
@@ -484,7 +620,7 @@ def run_operate(model_data, timings, backend, build_only):
             )
             # After iteration 1, warmstart = True, which should speed up the process
             # Note: Warmstart isn't possible with GLPK (dealt with later on)
-            _results = backend.solve_model(
+            _results, _opt = backend.solve_model(
                 backend_model,
                 solver=solver,
                 solver_io=solver_io,
@@ -501,10 +637,12 @@ def run_operate(model_data, timings, backend, build_only):
                 comment="Backend: iteration {}: solver finished running".format(i + 1),
             )
             # xarray dataset is built for each iteration
-            _termination = backend.load_results(backend_model, _results)
+            _termination = backend.load_results(backend_model, _results, _opt)
             terminations.append(_termination)
-
-            _results = backend.get_result_array(backend_model, model_data)
+            if _termination in ["optimal", "feasible"]:
+                _results = backend.get_result_array(backend_model, model_data)
+            else:
+                _results = xr.Dataset()
 
             # We give back the actual timesteps for this iteration and take a slice
             # equal to the window length
@@ -517,10 +655,13 @@ def run_operate(model_data, timings, backend, build_only):
             result_array.append(_results)
 
             # Set up initial storage for the next iteration
-            if "loc_techs_store" in model_data.dims.keys():
-                storage_initial = _results.storage.loc[
-                    {"timesteps": window_ends.index[i]}
-                ].drop("timesteps")
+            if "loc_techs_store" in model_data.dims.keys() and _termination in ["optimal", "feasible"]:
+                storage_initial = (
+                    _results.storage.loc[{"timesteps": window_ends.index[i]}].drop_vars(
+                        "timesteps"
+                    )
+                    / model_data.storage_cap
+                )
                 model_data["storage_initial"].loc[
                     storage_initial.coords
                 ] = storage_initial.values
@@ -529,10 +670,8 @@ def run_operate(model_data, timings, backend, build_only):
                 )
 
             # Set up total operated units for the next iteration
-            if "loc_techs_milp" in model_data.dims.keys():
-                operated_units = _results.operating_units.sum("timesteps").astype(
-                    np.int
-                )
+            if "loc_techs_milp" in model_data.dims.keys() and _termination in ["optimal", "feasible"]:
+                operated_units = _results.operating_units.sum("timesteps").astype(int)
                 model_data["operated_units"].loc[{}] += operated_units.values
                 backend_model.operated_units.store_values(
                     operated_units.to_series().dropna().to_dict()
@@ -548,6 +687,7 @@ def run_operate(model_data, timings, backend, build_only):
 
     if build_only:
         results = xr.Dataset()
+        _opt = None
     else:
         # Concatenate results over the timestep dimension to get a single
         # xarray Dataset of interest
@@ -567,4 +707,4 @@ def run_operate(model_data, timings, backend, build_only):
             comment="Backend: generated full solution array",
         )
 
-    return results, backend_model
+    return results, backend_model, _opt
