@@ -65,6 +65,8 @@ class AttrDict(dict):
 
     """
 
+    __name__ = "AttrDict"
+
     __getattr__ = dict.__getitem__
     __setattr__ = dict.__setitem__
 
@@ -171,7 +173,11 @@ class AttrDict(dict):
 
         """
         if isinstance(f, str) or isinstance(f, Path):
-            with open(f, "r", encoding="utf-8",) as src:
+            with open(
+                f,
+                "r",
+                encoding="utf-8",
+            ) as src:
                 loaded = cls(_yaml_load(src))
         else:
             loaded = cls(_yaml_load(f))
