@@ -14,6 +14,7 @@ import logging
 import warnings
 
 import numpy as np
+import xarray as xr
 import ruamel.yaml as ruamel_yaml
 
 from calliope.postprocess import results as postprocess_results
@@ -157,21 +158,8 @@ class Model(object):
             del model_data.attrs["_debug_data"]
 
         self._model_data = model_data
-        self.inputs = self._model_data.filter_by_attrs(is_result=0)
-        self.model_config = UpdateObserverDict(
-            initial_yaml_string=model_data.attrs.get("model_config", "{}"),
-            name="model_config",
-            observer=self._model_data,
-        )
-        self.run_config = UpdateObserverDict(
-            initial_yaml_string=model_data.attrs.get("run_config", "{}"),
-            name="run_config",
-            observer=self._model_data,
-        )
+        self._add_model_data_methods()
 
-        results = self._model_data.filter_by_attrs(is_result=1)
-        if len(results.data_vars) > 0:
-            self.results = results
         log_time(
             logger,
             self._timings,
@@ -179,6 +167,20 @@ class Model(object):
             comment="Model: loaded model_data",
         )
 
+    def _add_model_data_methods(self):
+        self.inputs = self._model_data.filter_by_attrs(is_result=0)
+        self.results = self._model_data.filter_by_attrs(is_result=1)
+        self.model_config = UpdateObserverDict(
+            initial_yaml_string=self._model_data.attrs.get("model_config", "{}"),
+            name="model_config",
+            observer=self._model_data,
+        )
+        self.run_config = UpdateObserverDict(
+            initial_yaml_string=self._model_data.attrs.get("run_config", "{}"),
+            name="run_config",
+            observer=self._model_data,
+        )
+
     def save_commented_model_yaml(self, path):
         """
         Save a fully built and commented version of the model to a YAML file
@@ -258,7 +260,7 @@ class Model(object):
                 "there exist non-uniform timesteps (e.g. from time masking)"
             )
 
-        results, self._backend_model, interface = run_backend(
+        results, self._backend_model, self._backend_model_opt, interface = run_backend(
             self._model_data, self._timings, **kwargs
         )
 
@@ -267,14 +269,11 @@ class Model(object):
             results = postprocess_results.postprocess_model_results(
                 results, self._model_data, self._timings
             )
-
-        for var in results.data_vars:
-            results[var].attrs["is_result"] = 1
-
-        self._model_data.update(results)
         self._model_data.attrs.update(results.attrs)
-
-        self.results = self._model_data.filter_by_attrs(is_result=1)
+        self._model_data = xr.merge(
+            [results, self._model_data], compat="override", combine_attrs="no_conflicts"
+        )
+        self._add_model_data_methods()
 
         self.backend = interface(self)
 
