@@ -16,6 +16,7 @@ import numpy as np
 
 from calliope.preprocess.util import constraint_exists, concat_iterable
 from calliope.preprocess import checks
+from calliope import exceptions
 
 
 def generate_constraint_sets(model_run):
@@ -418,8 +419,7 @@ def generate_constraint_sets(model_run):
         for name, data in model_run["group_constraints"].items()
         if data.get("exists", True)
     }
-    constraint_sets["constraint_groups"] = list(group_constraints.keys())
-
+    constraint_sets["group_constraints"] = set()
     for group_constraint_name, group_constraint in group_constraints.items():
         tech_groups = [
             [
@@ -446,8 +446,7 @@ def generate_constraint_sets(model_run):
             techs += [i + ":" + j for j in locs]
             techs.remove(i)
 
-        # If the group constraint defines its own techs, remove those that are
-        # not allowed (there is a warning for this in checks.py)
+        # If the group constraint defines its own techs, remove those that are not allowed
         techs = list(set(techs).intersection(allowed_techs))
 
         # All possible loc_techs for this constraint
@@ -458,9 +457,128 @@ def generate_constraint_sets(model_run):
         # Some loc_techs may not actually exist in the actual model,
         # so we must filter with actually exising loc_techs
         loc_techs = [i for i in loc_techs_all if i in sets.loc_techs]
+        default_group_config = checks.DEFAULTS.group_constraints.default_group
+        _constraints = {
+            k: v
+            for k, v in group_constraint.items()
+            if k not in ["locs", "techs", "exists"]
+        }
+        if any(
+            isinstance(default_group_config.get(_constraint, False), dict)
+            and "default_carrier" in default_group_config[_constraint].keys()
+            for _constraint in _constraints.keys()
+        ):
+            if len(_constraints) > 1:
+                raise exceptions.ModelError(
+                    "Can only handle one constraint in a group constraint if one of them is carrier-based"
+                )
+            _name, _config = list(_constraints.items())[0]
+            loc_tech_carrier_dict = _get_carrier_group_constraint_loc_techs(
+                loc_techs, locs, _config, _name, sets, constraint_sets
+            )
+            if any(len(val) == 0 for val in loc_tech_carrier_dict.values()):
+                exceptions.warn(
+                    f"Constraint group `{group_constraint_name}` will be completely ignored since there are no valid location::technology::carrier combinations"
+                )
+                continue
+            else:
+                for key, loc_tech_carriers in loc_tech_carrier_dict.items():
+                    constraint_sets[
+                        key.format(group_constraint_name)
+                    ] = loc_tech_carriers
 
-        constraint_sets[
-            "group_constraint_loc_techs_{}".format(group_constraint_name)
-        ] = loc_techs
-
+        else:
+            if len(loc_techs) == 0:
+                exceptions.warn(
+                    f"Constraint group `{group_constraint_name}` will be completely ignored since there are no valid location::technology combinations"
+                )
+                break
+            constraint_sets[
+                "group_constraint_loc_techs_{}".format(group_constraint_name)
+            ] = loc_techs
+        _add_to_group_constraint_mapping(
+            constraint_sets, group_constraint_name, list(_constraints.keys())
+        )
+    constraint_sets["group_constraints"] = list(constraint_sets["group_constraints"])
     return constraint_sets
+
+
+def _get_carrier_group_constraint_loc_techs(
+    loc_techs, locs, config, constraint_name, sets, constraint_sets
+):
+    flow = "con" if "_con_" in constraint_name else "prod"
+    if len(config.keys()) > 1:
+        raise exceptions.ModelError(
+            "Can only handle one carrier per group constraint that is carrier-based"
+        )
+    carrier = list(config.keys())[0]
+    if "net_import" in constraint_name:
+        _loc_tech_carriers = _get_net_import_loc_tech_carrier_subset(
+            loc_techs,
+            sets["loc_tech_carriers_con"] + sets["loc_tech_carriers_prod"],
+            carrier,
+            locs,
+        )
+    else:
+        _loc_tech_carriers = _get_loc_tech_carrier_subset(
+            loc_techs, sets[f"loc_tech_carriers_{flow}"], carrier, locs
+        )
+
+    if "share" in constraint_name:
+        lhs_loc_tech_carriers = _loc_tech_carriers
+        if "demand" in constraint_name or "import" in constraint_name:
+            rhs_loc_tech_carriers = _get_loc_tech_carrier_subset(
+                sets["loc_techs_demand"], sets["loc_tech_carriers_con"], carrier, locs
+            )
+        elif flow == "con":
+            rhs_loc_tech_carriers = _get_loc_tech_carrier_subset(
+                sets["loc_techs_demand"] + sets["loc_techs_conversion_all"],
+                sets["loc_tech_carriers_con"],
+                carrier,
+                locs,
+            )
+        elif flow == "prod":
+            rhs_loc_tech_carriers = _get_loc_tech_carrier_subset(
+                sets["loc_techs_supply_conversion_all"],
+                sets["loc_tech_carriers_prod"],
+                carrier,
+                locs,
+            )
+        return {
+            "group_constraint_loc_tech_carriers_{}_lhs": list(lhs_loc_tech_carriers),
+            "group_constraint_loc_tech_carriers_{}_rhs": list(rhs_loc_tech_carriers),
+        }
+    else:
+        return {"group_constraint_loc_tech_carriers_{}": list(_loc_tech_carriers)}
+
+
+def _get_loc_tech_carrier_subset(loc_techs, loc_tech_carriers, carrier, locs):
+    return set(
+        f"{loc_tech}::{carrier}"
+        for loc_tech in loc_techs
+        if f"{loc_tech}::{carrier}" in loc_tech_carriers
+        and loc_tech.split("::")[0] in locs
+    )
+
+
+def _get_net_import_loc_tech_carrier_subset(
+    loc_techs, loc_tech_carriers, carrier, locs
+):
+    return set(
+        f"{loc_tech}::{carrier}"
+        for loc_tech in loc_techs
+        if f"{loc_tech}::{carrier}" in loc_tech_carriers
+        and loc_tech.split(":")[-1] not in locs
+    )
+
+
+def _add_to_group_constraint_mapping(constraint_sets, group_name, constraint_names):
+    for constraint_name in constraint_names:
+        if (
+            f"group_names_{constraint_name}" in constraint_sets.keys()
+            and group_name not in constraint_sets[f"group_names_{constraint_name}"]
+        ):
+            constraint_sets[f"group_names_{constraint_name}"] += [group_name]
+        else:
+            constraint_sets[f"group_names_{constraint_name}"] = [group_name]
+        constraint_sets["group_constraints"].update([constraint_name])
