@@ -74,6 +74,7 @@ def build_model_data(model_run, debug=False):
 
     data = add_time_dimension(data, model_run)
 
+    data = update_dtypes(data)
     # Carrier information uses DataArray indexing in the function, so we merge
     # these directly into the main xarray Dataset
 
@@ -472,34 +473,33 @@ def group_constraints_to_dataset(model_run):
 
     group_constraints = model_run["group_constraints"]
 
-    for constr_name in model_run.sets["group_constraints"]:
+    for constr_name in model_run.constraint_sets["group_constraints"]:
         dims = ["group_names_" + constr_name]
         constr = checks.DEFAULTS.group_constraints.default_group.get(constr_name, None)
-        if isinstance(constr, dict):
-            if "default_carrier" in constr.keys():
-                dims.append("carriers")
-                data = [
-                    [
-                        group_constraints[i][constr_name].get(carrier, np.nan)
-                        for carrier in model_run.sets["carriers"]
+        if constr is not None:
+            group_names = model_run.constraint_sets.get(
+                f"group_names_{constr_name}", None
+            )
+            if isinstance(constr, dict):
+                if "default_carrier" in constr.keys():
+                    data = [
+                        list(group_constraints[i][constr_name].as_dict_flat().values())[
+                            0
+                        ]
+                        for i in group_names
                     ]
-                    for i in model_run.sets["group_names_" + constr_name]
-                ]
-            elif "default_cost" in constr.keys():
-                dims.append("costs")
-                data = [
-                    [
-                        group_constraints[i][constr_name].get(cost, np.nan)
-                        for cost in model_run.sets["costs"]
+                elif "default_cost" in constr.keys():
+                    dims.append("costs")
+                    data = [
+                        [
+                            group_constraints[i][constr_name].get(cost, np.nan)
+                            for cost in model_run.sets["costs"]
+                        ]
+                        for i in group_names
                     ]
-                    for i in model_run.sets["group_names_" + constr_name]
-                ]
-        elif constr is not None:
-            data = [
-                group_constraints[i][constr_name]
-                for i in model_run.sets["group_names_" + constr_name]
-            ]
-        else:  # Do nothing if it is an unknown constraint
+            else:
+                data = [group_constraints[i][constr_name] for i in group_names]
+        else:
             continue
 
         data_dict["group_" + constr_name] = {"dims": dims, "data": data}
@@ -552,3 +552,27 @@ def add_attributes(model_run):
     )
 
     return attr_dict
+
+
+def update_dtypes(model_data):
+    """
+    Update dtypes to not be 'Object', if possible.
+    Order of preference is: bool, int, float
+    """
+    for var_name, var in model_data.data_vars.items():
+        if var.dtype.kind in ["O", "U"]:
+            no_nans = var.where(var != "nan", drop=True)
+            model_data[var_name] = var.where(var != "nan")
+
+            if (no_nans.isin(["True", "False", "0", "1"]) | no_nans.isin([0, 1]) | no_nans.isin([True, False])).all():
+                # Turn to bool
+                model_data[var_name] = var.isin(["True", "1"]) | var.isin([1]) | var.isin([True])
+            else:
+                try:
+                    model_data[var_name] = var.astype(np.int, copy=False)
+                except (ValueError, TypeError):
+                    try:
+                        model_data[var_name] = var.astype(float, copy=False)
+                    except ValueError:
+                        continue
+    return model_data
