@@ -166,28 +166,6 @@ def generate_simple_sets(model_run):
     for k in model_run.model.get_key("group_share", {}).keys():
         sets.techlists.add(k)
 
-    # `constraint_groups` are the group names per constraint that is defined
-    # at a group level
-
-    sets.group_constraints = set()
-    group_constraints = AttrDict(
-        {
-            name: data
-            for name, data in model_run["group_constraints"].items()
-            if data.get("exists", True)
-        }
-    )
-    if len(group_constraints.keys()) > 0:
-        sets.group_constraints.update(
-            i.split(".")[1]
-            for i in group_constraints.as_dict_flat().keys()
-            if i.split(".")[1] not in ["techs", "locs"]
-        )
-        for constr in sets.group_constraints:
-            sets["group_names_" + constr] = set(
-                k for k, v in group_constraints.items() if constr in v.keys()
-            )
-
     return sets
 
 
@@ -355,8 +333,8 @@ def generate_loc_tech_sets(model_run, simple_sets):
     )
 
     # `supply_plus` technologies that specify a finite resource
-    sets.loc_techs_finite_resource_supply_plus = sets.loc_techs_finite_resource.intersection(
-        sets.loc_techs_supply_plus
+    sets.loc_techs_finite_resource_supply_plus = (
+        sets.loc_techs_finite_resource.intersection(sets.loc_techs_supply_plus)
     )
 
     # Technologies that define ramping constraints
