@@ -8,6 +8,7 @@ import pandas as pd
 import calliope
 from calliope.core.attrdict import AttrDict
 from calliope.test.common.util import build_test_model as build_model
+from calliope.test.common.util import check_error_or_warning
 
 
 def get_supply_conversion_techs(model):
@@ -80,9 +81,13 @@ class TestBuildGroupConstraints:
             len(
                 set(loc_techs[i])
                 - set(
-                    scenario_model._model_data[
-                        "group_constraint_loc_techs_" + group_names[i]
-                    ].values
+                    scenario_model._model_data.get(
+                        "group_constraint_loc_techs_" + group_names[i],
+                        scenario_model._model_data.get(
+                            "group_constraint_loc_tech_carriers_" + group_names[i],
+                            set(),
+                        ),
+                    ).values
                 )
             )
             == 0
@@ -1351,8 +1356,9 @@ class TestEnergyCapShareGroupConstraints:
         expensive_capacity = (
             capacity.loc[{"techs": "expensive_elec_supply"}].sum().item()
         )
-        assert expensive_capacity / capacity_supply_conversion_all.sum().item() == approx(
-            0.3
+        assert (
+            expensive_capacity / capacity_supply_conversion_all.sum().item()
+            == approx(0.3)
         )
 
     def test_energy_cap_share_min_max_supply_constraint(self, model_file):
@@ -1602,8 +1608,8 @@ class TestEnergyCapGroupConstraints:
         assert cheap_heat == approx(0.8333, rel=0.001)
         assert cheap_cool == approx(0.1667, rel=0.001)
         assert elec_to_heat_cool_linked == approx(1.6667, rel=0.001)
-        assert elec_to_heat_cool_unlinked == 0
-        assert elec_to_heat == 0
+        assert elec_to_heat_cool_unlinked == approx(0)
+        assert elec_to_heat == approx(0)
 
     # All technologies, but insufficient energy_cap_max for enough installed capacity to meet demand
     @pytest.mark.filterwarnings(
@@ -1670,7 +1676,12 @@ class TestNetImportShareGroupConstraints:
         assert net_imports == pytest.approx(-demand)
 
     def test_only_imports_with_wrong_carrier(self, results_for_scenario):
-        results = results_for_scenario("expensive-1,other-carrier")
+        with pytest.warns(calliope.exceptions.ModelWarning) as warn:
+            results = results_for_scenario("expensive-1,other-carrier")
+        assert check_error_or_warning(
+            warn,
+            "Constraint group `example_net_import_share_constraint` will be completely ignored",
+        )
         demand = self.retrieve_demand(results).sel(locs="1").item()
         net_imports = self.retrieve_net_imports(results).sel(locs="1").item()
         assert net_imports == pytest.approx(-demand)
@@ -1682,7 +1693,12 @@ class TestNetImportShareGroupConstraints:
 
     @pytest.mark.xfail(reason="One cannot define transmission techs explicitely.")
     def test_no_imports_explicit_tech(self, results_for_scenario):
-        results = results_for_scenario("expensive-1,no-net-imports-explicit-tech")
+        with pytest.warns(calliope.exceptions.ModelWarning) as warn:
+            results = results_for_scenario("expensive-1,no-net-imports-explicit-tech")
+        assert check_error_or_warning(
+            warn,
+            "Constraint group `example_net_import_share_constraint` will be completely ignored",
+        )
         net_imports = self.retrieve_net_imports(results).sel(locs="1").item()
         assert net_imports == pytest.approx(0)
 
@@ -1705,7 +1721,13 @@ class TestNetImportShareGroupConstraints:
         assert net_imports >= -0.2 * demand
 
     def test_ignores_imports_within_group(self, results_for_scenario):
-        results = results_for_scenario("expensive-1,ignores-imports-within-group")
+        with pytest.warns(calliope.exceptions.ModelWarning) as warn:
+            results = results_for_scenario("expensive-1,ignores-imports-within-group")
+        assert check_error_or_warning(
+            warn,
+            "Constraint group `example_net_import_share_constraint` will be completely ignored",
+        )
+
         demand = self.retrieve_demand(results).sel(locs="1").item()
         net_imports = self.retrieve_imports(results).sel(locs="1").item()
         assert net_imports == pytest.approx(-demand)
@@ -1811,8 +1833,13 @@ class TestCarrierProdGroupConstraints:
     # All conversion technologies with insufficient supply_max to use the
     # cheap direct heat/cooling supply techs
     def test_carrier_prod_max_non_conversion_all_constraint(self, model_file):
-        model = build_model(
-            model_file=model_file, scenario="carrier_prod_max_non_conversion_all"
+        with pytest.warns(calliope.exceptions.ModelWarning) as warn:
+            model = build_model(
+                model_file=model_file, scenario="carrier_prod_max_non_conversion_all"
+            )
+        assert check_error_or_warning(
+            warn,
+            "Constraint group `example_carrier_prod_constraint` will be completely ignored ",
         )
         model.run()
         prod = model.get_formatted_array("carrier_prod")
